@@ -15,7 +15,6 @@ use Catalyst qw/
     Unicode
     I18N
     Setenv
-    PageCache
     /;
 
 use Storable;
@@ -26,13 +25,12 @@ use DBIx::Class::ResultClass::HashRefInflator;
 use Encode ();
 use URI::Escape ();
 use MojoMojo::Formatter::Wiki;
-use Algorithm::IncludeExclude;
 use Module::Pluggable::Ordered
     search_path => 'MojoMojo::Formatter',
     except      => qr/^MojoMojo::Plugin::/,
     require     => 1;
 
-our $VERSION = '0.999033';
+our $VERSION = '0.999040';
 
 MojoMojo->config->{authentication}{dbic} = {
     user_class     => 'DBIC::Person',
@@ -49,28 +47,6 @@ MojoMojo->config->{'Plugin::Cache'}{backend} = {
     ),
 };
 
-MojoMojo->config(
-        'Plugin::PageCache' => {
-
-            # The expires is set in the .conf for easy edit outside of code.
-            # expires          => 300
-
-            # We don't set Cache-Control headers explicitly because
-            # firefox caches pre-login pages.
-            set_http_headers => 0,
-            auto_check_user  => 1,
-            auto_cache       => [
-                               '/.*',
-            ],
-            key_maker => sub {
-                my $c = shift;
-                return $c->stash->{path} . '.' . $c->req->path;
-            },
-            debug => 0 ,
-            cache_hook => 'cache_hook'
-        }
-);
-
 __PACKAGE__->config( authentication => {
     default_realm => 'members',
     use_session   => 1,
@@ -162,49 +138,6 @@ L<MojoMojo::Installation> to try it out yourself.
 
 =cut
 
-=head2 cache_ie_list
-
-Include/exclude list accessor
-
-=cut
-
-# include/exclude pages list to cache
-my $ie;
-$ie = Algorithm::IncludeExclude->new;
-$ie->include();
-# static files will be handled via web server or proxy cache control.
-$ie->exclude(qr{static});
-$ie->exclude(qr{attachment});
-$ie->exclude(qr{export});
-$ie->exclude('login');
-$ie->exclude('logout');
-$ie->exclude('edit');
-# Short cache time set in controller action for list and view
-#$ie->exclude('list');
-#$ie->exclude('recent');
-
-sub cache_ie_list {
-    return $ie;
-}
-
-=head2 cache_hook
-
-Dont cache if CATALYST_NOCACHE is set or if the path is excluded from,
-based on the value C<cache_ie_list>.
-
-=cut
-sub cache_hook {
-  my ( $c ) = @_;
-
-  if ( $ENV{CATALYST_NOCACHE} ||
-       $c->response->location ||
-       ! $c->cache_ie_list->evaluate($c->req->path)
-     ) {
-    return 0; # Don't cache
-  }
-  return 1;   # Cache
-}
-
 sub ajax {
     my ($c) = @_;
     return $c->req->header('x-requested-with')
@@ -400,14 +333,6 @@ sub base_uri {
     return URI->new( $c->req->base );
 }
 
-# format for unicode template use.
-
-sub unicode {
-    my ( $c, $string ) = @_;
-    utf8::decode($string);
-    return $string;
-}
-
 =head2 uri_for
 
 Override $c->uri_for to append path, if a relative path is used.
@@ -543,6 +468,11 @@ sub get_permissions_data {
     # We have two options here - if we are caching, we will load everything and cache it.
     # If we are not - then we load just the bits we need.
     if ( !$permdata ) {
+        # Initialize $permdata as a reference or we end up with an error
+        # when we try to dereference it further down.  The error we're avoiding is:
+        # Can't use string ("") as a HASH ref while "strict refs"
+        $permdata = {};
+        
         ## either the data hasn't been loaded, or it's expired since we used it last.
         ## so we need to reload it.
         my $rs =
