@@ -1,7 +1,6 @@
 package MojoMojo::Controller::PageAdmin;
 use warnings;
 use strict;
-use Data::Dumper;
 use parent 'Catalyst::Controller::HTML::FormFu';
 use Syntax::Highlight::Engine::Kate;
 
@@ -15,7 +14,7 @@ See L<MojoMojo>
 
 =head1 DESCRIPTION
 
-methods for administration of pages.
+Methods for updating pages: edit, rollback, permissions change, rename.
 
 =head1 METHODS
 
@@ -25,10 +24,24 @@ Check that user is logged in and has rights to this page.
 
 =cut
 
+=head2 unauthorized
+
+Private action to return a 403 with an explanatory template.
+
+=cut
+
+sub unauthorized : Private {
+    my ( $self, $c, $operation ) = @_;
+    $c->stash->{template} = 'message.tt';
+    $c->stash->{message} ||= $c->loc('No permissions to x this page', $operation || $c->loc('update'));
+    $c->response->status(403) unless $c->response->status;  # 403 Forbidden
+    return 0;
+}
+
 sub auto : Private {
     my ( $self, $c ) = @_;
     $c->forward('/user/login')
-      if $c->req->params->{pass}
+        if $c->req->params->{pass}  # XXX use case?
           && !$c->stash->{user};
 
     # everyone can edit with anon mode enabled.
@@ -36,9 +49,7 @@ sub auto : Private {
     my $user = $c->stash->{user};
     return 1 if $user && $user->can_edit( $c->stash->{path} );
     return 1 if $user && !$c->pref('restricted_user');
-    $c->stash->{template} = 'message.tt';
-    $c->stash->{message}  = $c->loc('No permissions to edit this page');
-    return 0;
+    $c->detach('unauthorized', [$c->loc('edit')]);
 }
 
 =head2 edit
@@ -57,7 +68,7 @@ sub edit : Global FormConfig {
     my $stash = $c->stash;
     $stash->{template} = 'page/edit.tt';
 
-    my $user = $c->user_exists ? $c->user->obj->id : 1;    # Anon edit
+    my $user_id = $c->user_exists ? $c->user->obj->id : 1;    # Anon edit
 
     my ( $path_pages, $proto_pages ) = @$stash{qw/ path_pages proto_pages /};
 
@@ -72,33 +83,38 @@ sub edit : Global FormConfig {
     # proto_pages, depending on whether or not the page already exists
     my $page = (
           @$proto_pages > 0
-        ? $proto_pages->[ @$proto_pages - 1 ]
-        : $path_pages->[ @$path_pages - 1 ]
+        ? $proto_pages->[-1]
+        : $path_pages->[-1]
     );
 
     # this should never happen!
-    $c->detach('/default') unless $page;
+    $c->detach('/default') unless $page;  # TODO there is no default action, BTW
     @$stash{qw/ path_pages proto_pages /} = ( $path_pages, $proto_pages );
 
     my $perms =
       $c->check_permissions( $stash->{'path'},
         ( $c->user_exists ? $c->user->obj : undef ) );
     my $permtocheck = ( @$proto_pages > 0 ? 'create' : 'edit' );
-    my $loc_permtocheck=$permtocheck eq 'create'?$c->loc('create'):$c->loc('edit');
+    my $loc_permtocheck = $permtocheck eq 'create'
+      ? $c->loc('create')
+      : $c->loc('edit');
+      
+    # TODO this should be caught in the auto action. To reproduce, disable "Edit allowed by default"
+    # in Site settings, then go to /.edit
     if ( !$perms->{$permtocheck} ) {
         my $name = ref($page) eq 'HASH' ? $page->{name} : $page->name;
         $stash->{message} =
-          $c->loc( 'Permission Denied to x x', [ $loc_permtocheck, $name ] );
-        $stash->{template} = 'message.tt';
-        return;
+          $c->loc( 'Permission denied to x x', [ $loc_permtocheck, $name ] );
+        $c->detach('unauthorized');
     }
-    if ( $user == 1 && !$c->pref('anonymous_user') ) {
-        $c->stash->{message} ||= $c->loc('Anonymous Edit disabled');
-        return;
+    # TODO in the use case above, the message below should be displayed. However, that never happens
+    if ( $user_id == 1 && !$c->pref('anonymous_user') ) {
+        $c->stash->{message} = $c->loc('Anonymous edit disabled');
+        $c->detach('unauthorized');
     }
 
     # for anonymous users, use CAPTCHA, if enabled
-    if ( $user == 1 && $c->pref('use_captcha') ) {
+    if ( $user_id == 1 && $c->pref('use_captcha') ) {
         my $captcha_lang = $c->session->{lang} || $c->pref('default_lang') ;
         $c->stash->{captcha} = $form->element({
             type => 'reCAPTCHA',
@@ -116,18 +132,17 @@ sub edit : Global FormConfig {
 
     if ( $form->submitted_and_valid ) {
 
-
         my $valid = $form->params;
-        $valid->{creator} = $user;
+        $valid->{creator} = $user_id;
 
         if (@$proto_pages) {    # page doesn't exist yet
 
             $path_pages = $c->model('DBIC::Page')->create_path_pages(
                 path_pages  => $path_pages,
                 proto_pages => $proto_pages,
-                creator     => $user,
+                creator     => $user_id,
             );
-            $page = $path_pages->[ @$path_pages - 1 ];
+            $page = $path_pages->[-1];
         }
 
         $stash->{content} = $page->content;
@@ -154,6 +169,13 @@ sub edit : Global FormConfig {
                 $c->loc('END OF CONFLICT'));
             return;
         }
+        # Format content body and store the result in content.precompiled 
+        # This speeds up MojoMojo page rendering on /.view actions
+        my $precompiled_body = $valid->{'body'};
+        MojoMojo->call_plugins( 'format_content', \$precompiled_body, $c, $page );
+        
+        # Make precompiled empty when we have any of: redirect, comment or include
+        $valid->{'precompiled'} = $c->stash->{precompile_off} ? '' : $precompiled_body;
 
         $page->update_content(%$valid);
 
@@ -169,10 +191,6 @@ sub edit : Global FormConfig {
         my $redirect = $c->uri_for( $c->stash->{path} );
         if ( $form->params->{submit} eq $c->localize('Save') ) {
             $redirect .= '.edit';
-            if ( $c->req->params->{split} &&
-                 $c->req->params->{'split'} eq 'vertical' ) {
-                $redirect .= '?split=vertical';
-            }
         }
         $c->res->redirect($redirect);
     }
@@ -288,7 +306,17 @@ This action will revert a page to a older revision.
 
 sub rollback : Global {
     my ( $self, $c, $page ) = @_;
+    if ($c->req->method ne 'POST') {
+        # general error - we want a POST
+        $c->res->status(400);
+        $c->detach('unauthorized', [$c->loc('rollback')]);
+    }
+    
     if ( $c->req->param('rev') ) {
+        # TODO this needs to do a proper versioned rollback, via
+        # $page->add_version( content_version => $c->req->param('rev')
+        # The problem is that the page_version table doesn't have a content_version field
+        # We could cannibalize the parent_version field, which is dummily always '1'
         $c->stash->{page}->content_version( $c->req->param('rev') );
         $c->stash->{page}->update;
         undef $c->req->params->{rev};
